package com.trust.ywx.customer.common.nacos;

import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.config.annotation.NacosConfigListener;
import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.alibaba.nacos.api.config.listener.AbstractListener;
import com.alibaba.nacos.api.exception.NacosException;
import com.alibaba.nacos.client.config.listener.impl.PropertiesListener;
import com.fiftyonetrust.common.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;

import javax.annotation.Resource;
import java.util.Properties;

/**
 * <pre></pre>
 *
 * @文件名称： ConfigNacos.java
 * @包 路  径： com.trust.ywx.customer.common.nacos
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/29 15:39
 * @Modify：
 */
@Slf4j
@Configuration
public class NacosServer {

    @Resource
    private NacosConfig nacosConfig;

    private FirmNacosConfig firmNacosConfig;
    @Resource
    private ConfigService configService;

    private static final String DATA_ID = "com.trust.ywx.customer.common.nacos.NacosConfig";
    private static final String GROUP = "ywx-customer";


    @Bean
    public ConfigService buildCache() throws NacosException {
        Properties properties = nacosConfig.getPropertie();
        configService = NacosFactory.createConfigService(properties);
        configService.addListener(DATA_ID, GROUP, new AbstractListener() {
            @Override
            public void receiveConfigInfo(String config) {
                if(StringUtils.isNotEmpty(config)){
                    firmNacosConfig = JsonUtil.of(config,FirmNacosConfig.class);
                }
            }
        });
        String config = configService.getConfig(DATA_ID,GROUP,5000);
        firmNacosConfig = JsonUtil.of(config,FirmNacosConfig.class);

        return configService;
    }

    public FirmNacosConfig getFirmNacosConfig() {
        if(firmNacosConfig==null){
            try {
                String config = configService.getConfig(DATA_ID,GROUP,5000);
                firmNacosConfig = JsonUtil.of(config,FirmNacosConfig.class);
            } catch (NacosException e) {
                log.error("nacos get config error:{}",e.getMessage());
            }
        }
        return firmNacosConfig;
    }
}
