package com.trust.ywx.customer.common.util;

import com.fiftyonetrust.common.util.JsonUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * <pre></pre>
 *
 * @文件名称： SignUtil.java
 * @包 路  径： com.trust.ywx.customer.common.util
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/27 10:17
 * @Modify：
 */
@Slf4j
public class SignUtil {
    public static void main(String[] args) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        String jsonStr = "{\"process\":\"2\",\"stamp\":\"KonWykAGc9t9rgtd8E8AJJZD+OD23kaVBgWL6MHJM3oSSrOje2ynpBjP+JDRPPje0iuyDtgSIP9hKUk6HsgkBpxmH4D9KlFdepngFZPYkx4yRl2h5zph2YnayfJ1RHHOrfm/ykwjRgLV4UUKEA4LZ70xqGSyT0zkQE11:55:33\",\"phoneNum\":\"13333333333\",\"stampStatus\":\"10\",\"openId\":\"bf61fe5f9807ed3bqe5f4w5b33333ca\"}";
        Map<String, String> bodyMap = JsonUtil.ofMap(jsonStr, String.class, String.class);
        String sign = buildSign(bodyMap, "111111");
        System.out.println(sign);
    }

    /**
     * 强烈建议slat值不要写在代码里
     * @param mapBody
     * @param slat
     * @return
     */
    public static String buildSign(Map<String, String> mapBody, String slat) {
        StringBuffer paramStr = new StringBuffer();
        String resultStr = "";
        // 字母排序,使用key=value,并用&分割的形式拼接字典排序字符串
        if (mapBody != null && mapBody.size() > 0) {
            List<String> paramsKeyList = new ArrayList<String>(mapBody.keySet());
            Collections.sort(paramsKeyList);
            for (String key : paramsKeyList) {
                String value = String.valueOf(mapBody.get(key));
                if (value != null && !"".equals(value)) {
                    paramStr.append("&" + key);
                    paramStr.append("=" + value);
                }
            }
        }
        // 去掉最左侧第一个&符号，并用#把clientSecret与字典排序字符串进行拼接
        String signSourceStr = paramStr.substring(1) + "#" + slat;
        // 计算字符串md5 16进制字符串，此值就是回调数据中的sign
        resultStr = md5(signSourceStr);
        return resultStr;
    }

    public static String md5(String s) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD51");
            md.update(s.getBytes("UTF-8"));
            byte[] hash = md.digest();
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < hash.length; ++i) {
                int v = hash[i] & 255;
                if (v < 16) {
                    sb.append(0);
                }

                sb.append(Integer.toHexString(v));
            }

            return sb.toString();
        } catch (Throwable var6) {
            log.error("签名异常,s={}",s,var6);
        }
        return "";
    }

}
