package com.trust.ywx.customer.common.util;

import org.apache.commons.lang3.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 公用参数校验工具类。
 *
 * @Author hezhao
 * @Date 2015年3月25日
 */
public class ValidateUtil {

    private static final Pattern pip4 = Pattern.compile("\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}");// ip地址匹配192.168.1.3
    private static final Pattern pdomain = Pattern.compile("[a-zA-Z0-9-\\u4e00-\\u9fa5]+(.[a-zA-Z0-9-]+)+");
    private static final Pattern purl = Pattern.compile("(http|https):\\/\\/[\\w\\-_]+(\\.[\\w\\-_]+)+([\\w\\-\\.,@?^=%&amp;:/~\\+#]*[\\w\\-\\@?^=%&amp;/~\\+#])?");
    private static final Pattern pserialno = Pattern.compile("[a-z0-9-_]+");
    private static final String DOMAINS = ".com.cn|.net.cn|.org.cn|.gov.cn|.com|.net|.tv|.gd|.org|.cc|.vc|.mobi|.cd|.info|.name|.asia|.hk|.me|.la|.sh|.biz|.li|.kr|.in|.us|.io|.ac.cn|.bj.cn|.sh.cn|.tj.cn|.cq.cn|.he.cn|.sx.cn|.nm.cn|.ln.cn|.jl.cn|.hl.cn|.js.cn|.zj.cn|.ah.cn|.fj.cn|.jx.cn|.sd.cn|.ha.cn|.hb.cn|.hn.cn|.gd.cn|.gx.cn|.hi.cn|.sc.cn|.gz.cn|.yn.cn|.xz.cn|.sn.cn|.gs.cn|.qh.cn|.nx.cn|.xj.cn|.tw.cn|.hk.cn|.mo.cn|.cn";

    private ValidateUtil() {
        // 私有类构造方法
    }

    /**
     * 校验字符串是否是英文字母，不分大小写
     *
     * @author yuanchangjian
     * @return
     */
    public static boolean isEnglish(String value) {
        if (StringUtils.isBlank(value)) {
            return false;
        }
        Pattern pattern = Pattern.compile("^[A-Za-z]+$");
        Matcher m = pattern.matcher(value);
        return m.matches();
    }

    /**
     * 校验电话号码（座机）
     *
     * @author yuanchangjian
     * @return
     */
    public static boolean checkPhone(String value) {
        if (StringUtils.isBlank(value)) {
            return false;
        }
        Pattern pattern = Pattern.compile("^((\\d{3,4})\\-(\\d{8})|(\\d{4})\\-(\\d{7}))$");
        Matcher m = pattern.matcher(value);
        return m.matches();

    }

    /**
     * 校验电话号码（手机）
     *
     * @author yuanchangjian
     * @return
     */
    public static boolean checkMobile(String value) {
        if (StringUtils.isBlank(value)) {
            return false;
        }
        Pattern pattern = Pattern.compile("^1(4[0-9]|3[0-9]|5[0-9]|8[0-9]|7[0-9])\\d{8}$");
        Matcher m = pattern.matcher(value);
        return m.matches();
    }

    /**
     * 运营商号段如下： 中国联通号码：130、131、132、145（无线上网卡）、155、156、185（iPhone5上市后开放）、186、176（4G号段）、 175（2015年9月10日正式启用，暂只对北京、上海和广东投放办理）
     * 中国移动号码：134、135、136、137、138、139、147（无线上网卡）、150、151、152、157、158、159、182、183、187、188、178 中国电信号码：133、153、180、181、189、177、173、149 虚拟运营商：170、1718、1719 手机号前3位的数字包括： 1 :1 2 :3,4,5,7,8 3
     * :0,1,2,3,4,5,6,7,8,9 总结： 目前java手机号码正则表达式有： a :"^1[3|4|5|7|8][0-9]\\d{4,8}$" 一般验证情况下这个就可以了 b :"^((13[0-9])|(14[5|7])|(15([0-3]|[5-9]))|(17[013678])|(18[0,5-9]))\\d{8}$"
     * Pattern和Matcher详解（字符串匹配和字节码）http://blog.csdn.net/u010700335/article/details/44616451
     */
    public static boolean checkMobileNo(String mobileNo) {
        if (StringUtils.isBlank(mobileNo)) {
            return false;
        }
        String regex = "^((13[0-9])|(14[5|7])|(15([0-3]|[5-9]))|(17[013678])|(18[0,5-9]))\\d{8}$";
        boolean isMatch = false;//默认不合格
        try {
            if (mobileNo.length() != 11) {
                isMatch = false;
            } else {
                Pattern p = Pattern.compile(regex);
                Matcher m = p.matcher(mobileNo);
                isMatch = m.matches();
            }
        } catch (Exception e) {
            isMatch = false;
            e.printStackTrace();
        }
        return isMatch;
    }

    /**
     * 校验电子邮箱
     *
     * @author yuanchangjian
     * @return
     */
    public static boolean checkEmail(String value) {
        if (StringUtils.isBlank(value)) {
            return false;
        }
        Pattern pattern = Pattern.compile("^([a-zA-Z0-9]+[_|\\_|\\.]?)*[a-zA-Z0-9]+@([a-zA-Z0-9]+[_|\\_|\\.]?)*[a-zA-Z0-9]+\\.[a-zA-Z]{2,3}$");
        Matcher m = pattern.matcher(value);
        return m.matches();
    }

    /**
     * 是否是url地址
     *
     * @param str http://www.oppo.com url地址
     * @return 是否是url地址
     */
    public static boolean checkURL(String str) {
        if (StringUtils.isBlank(str)) {
            return false;
        }
        Matcher m = purl.matcher(str);
        return m.matches();
    }

    /**
     * 是否是主域名如 baidu.com
     *
     * @param str 目标字符串
     * @return 是否是主域名
     */
    public static boolean checkMainDomain(String str) {
        if (StringUtils.isBlank(str)) {
            return false;
        }
        String[] domains = DOMAINS.split("\\|");
        for (String s : domains) {
            if (null != s && !s.trim().equals("") && str.indexOf(s) > 0) {
                str = str.replace(s, "");
                if (checkSerialNo(str)) {
                    return true;
                } else {
                    return false;
                }
            }
        }
        return false;
    }

    public static boolean checkDomain(String s) {
        if (StringUtils.isBlank(s)) {
            return false;
        }
        Matcher m = pdomain.matcher(s);
        return m.matches();
    }


    /**
     * 是不是IP地址
     *
     * @param s 目标字符串
     * @return 是不是IP地址
     */
    public static boolean checkIP(String s) {
        if (StringUtils.isBlank(s)) {
            return false;
        }
        Matcher m = pip4.matcher(s);
        return m.matches();
    }

    /**
     * 校验编号, 只能是数字,字母, 下划线, 减号
     *
     * @param str 目标字符串 例如：http://www.oppo.com
     * @return 是否是编号
     */
    public static boolean checkSerialNo(String str) {
        if (StringUtils.isBlank(str)) {
            return false;
        }
        str = str.toLowerCase().trim();

        Matcher isSerialNo = pserialno.matcher(str);
        return isSerialNo.matches();
    }

}