package com.trust.ywx.customer.dao;

import com.github.pagehelper.PageHelper;
import com.trust.ywx.customer.common.enu.FirmStatusEnum;
import com.trust.ywx.customer.common.model.PageBean;
import com.trust.ywx.customer.common.model.PageBeanUtil;
import com.trust.ywx.customer.common.model.manager.request.FirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchFirmParam;
import com.trust.ywx.customer.dao.mapper.FirmInfoMapper;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.dao.po.FirmInfoExample;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <pre></pre>
 *
 * @文件名称： FirmDao.java
 * @包 路  径： com.trust.ywx.customer.dao
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户查询DAO
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/16 10:31
 * @Modify：
 */
@Repository
public class FirmDao {

    @Resource
    private FirmInfoMapper firmInfoMapper;

    public List<FirmInfo> getListByParam(SearchFirmParam param) {
        FirmInfoExample example = new FirmInfoExample();
        FirmInfoExample.Criteria where = example.createCriteria();
        where.andStatusNotEqualTo(FirmStatusEnum.FIRM_SUB_DELETE.getCode());
        // 增加项目状态的判断
        if (null != param.getProjectStatus()) {
            where.andProjectStatusEqualTo(param.getProjectStatus());
        }
        // 增加实施人员的判断
        if (StringUtils.isNotEmpty(param.getTechId())) {
            where.andTechPersonIdEqualTo(param.getTechId());
        }
        // 增加销售人员的判断
        if (StringUtils.isNotEmpty(param.getSaleId())) {
            where.andSalePersonIdEqualTo(param.getSaleId());
        }
        // 增加第三方平台客户id的判断
        if (StringUtils.isNotEmpty(param.getThirdCustomerId())) {
            where.andThirdCustomerIdEqualTo(param.getThirdCustomerId());
        }
        return firmInfoMapper.selectByExample(example);
    }
    public List<FirmInfo> getListByOrgCode(String orgCode) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andFirmOrgCodeEqualTo(orgCode);
        return firmInfoMapper.selectByExample(example);
    }

    public List<FirmInfo> getListByFirmName(String firmName) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andFirmNameEqualTo(firmName);
        return firmInfoMapper.selectByExample(example);
    }

    public PageBean<FirmInfo> getListByIds(List<String> clientIdList, int pageNo,int pageSize) {
        PageHelper.startPage(pageNo, pageSize);

        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdIn(clientIdList).andStatusNotEqualTo(FirmStatusEnum.FIRM_SUB_DELETE.getCode());
        List list = firmInfoMapper.selectByExample(example);
        return new PageBean<>(new ArrayList<>(list),list);
    }

    public PageBean<FirmInfo> getList(int pageNo,int pageSize) {
        PageHelper.startPage(pageNo, pageSize);
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andStatusNotEqualTo(FirmStatusEnum.FIRM_SUB_DELETE.getCode());
        List list = firmInfoMapper.selectByExample(example);
        return new PageBean<>(new ArrayList<>(list),list);
    }
    /**
     * 根据客户Id查询客户信息
     * 已删除的客户不对外暴露
     *
     * @param clientId
     * @return
     */
    public FirmInfo getFrimByClientId(String clientId) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.selectOneByExample(example);
    }

    /**
     * 根据客户Id查询客户信息
     * 已删除的客户不对外暴露
     *
     * @param clientId
     * @return
     */
    public FirmInfo getFirmBLOBsByClientId(String clientId) {
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.selectOneByExampleWithBLOBs(example);
    }


    public int insert(FirmInfo firmInfo) {
        firmInfo.setCreateTime(null);
        firmInfo.setUpdateTime(null);
        return firmInfoMapper.insertSelective(firmInfo);
    }

    public int update(FirmInfo firmInfo) {
        firmInfo.setUpdateTime(null);
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(firmInfo.getClientId());
        return firmInfoMapper.updateByExampleSelective(firmInfo, example);
    }

    /**
     * 根据客户id删除客户
     *
     * @param clientId
     * @return
     */
    public int deleteByClientId(String clientId) {
        FirmInfo firmInfo = new FirmInfo();
        firmInfo.setUpdateTime(null);
        firmInfo.setStatus(FirmStatusEnum.FIRM_SUB_DELETE.getCode());
        FirmInfoExample example = new FirmInfoExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return firmInfoMapper.updateByExampleSelective(firmInfo, example);
    }
}
