package com.trust.ywx.customer.dao;

import com.github.pagehelper.PageHelper;
import com.trust.ywx.customer.common.enu.TempFirmStatusEnum;
import com.trust.ywx.customer.common.model.manager.response.TempFirmDto;
import com.trust.ywx.customer.common.model.query.request.SearchTempFirmParam;
import com.trust.ywx.customer.common.model.PageBean;
import com.trust.ywx.customer.dao.mapper.FirmInfoTempMapper;
import com.trust.ywx.customer.dao.po.FirmInfoTemp;
import com.trust.ywx.customer.dao.po.FirmInfoTempExample;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <pre></pre>
 *
 * @文件名称： AuditFirmDao.java
 * @包 路  径： com.trust.ywx.customer.dao
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/21 11:40
 * @Modify：
 */
@Repository
public class TempFirmDao {
    @Resource
    FirmInfoTempMapper tempMapper;

    private List<String> statusList = Arrays.asList(TempFirmStatusEnum.TEMP_UNAUTHEN.getCode()+"",TempFirmStatusEnum.TEMP_REFUSE.getCode()+"");

    public FirmInfoTemp getByClientId(String clientId){
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andClientIdEqualTo(clientId).andStatusIn(statusList);
        return tempMapper.selectOneByExampleWithBLOBs(example);
    }

    public int auditPass(String clientId){
        // 将临时客户修改为已审核！
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andClientIdEqualTo(clientId).
                andStatusEqualTo(TempFirmStatusEnum.TEMP_UNAUTHEN.getCode()+"");

        FirmInfoTemp record = new FirmInfoTemp();
        record.setStatus(TempFirmStatusEnum.TEMP_PASS.getCode()+"");
        return tempMapper.updateByExampleSelective(record,example);
    }

    public int auditReject(String clientId){
        // 将临时客户修改为审核拒绝！
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andClientIdEqualTo(clientId).
                andStatusEqualTo(TempFirmStatusEnum.TEMP_UNAUTHEN.getCode()+"");

        FirmInfoTemp record = new FirmInfoTemp();
        record.setStatus(TempFirmStatusEnum.TEMP_REFUSE.getCode()+"");
        return tempMapper.updateByExampleSelective(record,example);
    }

    public List<FirmInfoTemp> getListByFirmName(String firmName) {
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andFirmNameEqualTo(firmName);
        return tempMapper.selectByExample(example);
    }

    public int insert(FirmInfoTemp temp) {
        temp.setCreateTime(null);
        temp.setUpdateTime(null);
        return tempMapper.insertSelective(temp);
    }
    public int update(FirmInfoTemp temp) {
        temp.setUpdateTime(null);
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andClientIdEqualTo(temp.getClientId());
        return tempMapper.updateByExampleSelective(temp,example);
    }

    public int deleteByClientId(String clientId) {
        FirmInfoTempExample example = new FirmInfoTempExample();
        example.createCriteria().andClientIdEqualTo(clientId);
        return tempMapper.deleteByExample(example);
    }

    public PageBean<TempFirmDto> getListByParam(SearchTempFirmParam param) {
        PageHelper.startPage(param.getPageNo(),param.getPageSize());
        FirmInfoTempExample example = new FirmInfoTempExample();
        FirmInfoTempExample.Criteria where = example.createCriteria();

        if(StringUtils.isNotBlank(param.getClientId())){
            where.andClientIdEqualTo(param.getClientId());
        }
        if(StringUtils.isNotBlank(param.getFirmName())){
            where.andFirmNameEqualTo(param.getFirmName());
        }
        if(null != param.getStatus()){
            where.andStatusEqualTo(param.getStatus()+"");
        }else{
            // 查询待审核和审核拒绝的
            where.andStatusIn(statusList);
        }
        example.setOrderByClause(FirmInfoTemp.Column.createTime.desc());

        List<FirmInfoTemp> list = tempMapper.selectByExample(example);
        return new PageBean(new ArrayList<>(list),list);
    }
}
