package com.trust.ywx.customer.service.manager.impl;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.SendResult;
import com.aliyun.openservices.ons.api.bean.ProducerBean;
import com.fiftyonetrust.common.model.exception.CodeException;
import com.trust.ywx.customer.common.CommonConstants;
import com.trust.ywx.customer.common.enu.*;
import com.trust.ywx.customer.common.model.manager.request.FirmParam;
import com.trust.ywx.customer.common.model.manager.response.FirmBaseDto;
import com.trust.ywx.customer.common.util.FirmPropUtil;
import com.trust.ywx.customer.commons.UniqueIdUtils;
import com.trust.ywx.customer.dao.FirmDao;
import com.trust.ywx.customer.dao.FirmExtendDao;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.service.manager.FirmManagerService;
//import com.trust.ywx.customer.service.mq.config.MqConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <pre></pre>
 *
 * @文件名称： FirmManagerServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.manager.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户管理实现类
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/20 12:50
 * @Modify：
 */
@Slf4j
@Service("firmManagerService")
public class FirmManagerServiceImpl implements FirmManagerService {
    @Resource
    TransactionTemplate transactionTemplate;
    @Resource
    FirmDao firmDao;
    @Resource
    FirmExtendDao firmExtendDao;
    @Resource
    private ProducerBean producer;
//    @Resource
//    private MqConfig mqConfig;

    @Override
    public FirmBaseDto init(String clientId) {
        FirmBaseDto dto = new FirmBaseDto();
        dto.setClientId(UniqueIdUtils.generateClientId());
        dto.setAppSecret(UniqueIdUtils.generateAppSecret());
        return dto;
    }

    @Override
    public boolean add(FirmParam param) throws CodeException {
        // 参数校验
        checkParam(param, true);
        // 业务校验
        checkAddBiz(param);
        // 执行插入
        return doAddTrust(param);
    }

    private void checkParam(FirmParam param, boolean isAdd) throws CodeException {
        if (StringUtils.isBlank(param.getClientId())) {
            throw new CodeException(FirmErrEum.FIRM_CLIENT_ID_NULL.getStatus(),FirmErrEum.FIRM_CLIENT_ID_NULL.getMessage());
        }
        if (StringUtils.isBlank(param.getAppSecret())) {
            throw new CodeException(FirmErrEum.FIRM_APP_SECRET_NULL.getStatus(),FirmErrEum.FIRM_APP_SECRET_NULL.getMessage());
        }
        if (StringUtils.isBlank(param.getFirmName())) {
            throw new CodeException(FirmErrEum.FIRM_NAME_NULL.getStatus(),FirmErrEum.FIRM_NAME_NULL.getMessage());
        }
        if (StringUtils.isNotBlank(param.getFirmName()) && param.getFirmName().length() > 30) {
            throw new CodeException(FirmErrEum.FIRM_NAME_TOO_LONG.getStatus(),FirmErrEum.FIRM_NAME_TOO_LONG.getMessage());
        }
//        if (StringUtils.isBlank(param.getFirmPerson())) {
//            throw new CodeException(FirmErrEum.FIRM_PERSON);
//        }
//        if (!ValidateUtil.checkMobile(param.getFirmPhone())) {
//            throw new CodeException(FirmErrEum.FIRM_PHONE);
//        }

        if (StringUtils.isBlank(param.getFirmOrgCode())) {
            throw new CodeException(FirmErrEum.CLIENT_ORG_CODE_NULL.getStatus(),FirmErrEum.CLIENT_ORG_CODE_NULL.getMessage());
        }

        if (CommonStatusEnum.YES.getCode().equals(param.getSmsCustomize()) && StringUtils.isNotBlank(param.getSmsCustomizeContent())
                && param.getSmsCustomizeContent().length() > 101) {
            throw new CodeException(FirmErrEum.FIRM_SMS_CUSTOMIZECONTENT_TOOLONG.getStatus(),FirmErrEum.FIRM_SMS_CUSTOMIZECONTENT_TOOLONG.getMessage());
        }
//        if (!StringUtils.isBlank(param.getDoctorProductType()) && !publicProductType.equals("SDK")
//                && !publicProductType.equals("H5")) {
//            throw new CodeException(GlobalExcpEnum.PARAM_ERROR);
//        }
        /*cachannel校验*/
        if (param.getCaChannel() != null && null == CaChannelEnum.getDes(param.getCaChannel())) {
            throw new CodeException(FirmErrEum.FIRM_CA_CHANNEL_ERR.getStatus(),FirmErrEum.FIRM_CA_CHANNEL_ERR.getMessage());
        }

        if (StringUtils.isNotBlank(param.getAppIcon())) {
            if (param.getAppIcon().length() > 64 * 1024) {
                throw new CodeException(FirmErrEum.FIRM_ICON_TOOBIG.getStatus(),FirmErrEum.FIRM_ICON_TOOBIG.getMessage());
            }
        }

        if (param.getTechSupportInfo() != null && param.getTechSupportInfo().length() > 300) {
            throw new CodeException(FirmErrEum.SUPPORT_INFO_TOOLONG.getStatus(),FirmErrEum.SUPPORT_INFO_TOOLONG.getMessage());
        }
    }

    @Override
    public boolean update(FirmParam param) throws CodeException {
        // 参数校验
        checkParam(param, true);
        // 业务校验
        checkUpdateBiz(param);
        // 执行插入
        boolean result =  doUpdateTrust(param);
        // 消息通知消费者更新客户缓存信息
        notifyComsumer(param);
        return result;
    }

    private void notifyComsumer(FirmParam param) {
        JSONObject json = new JSONObject();
        FirmBaseDto baseDto = buildBaseDto(param);
        Map<String,String> extendMap = FirmPropUtil.convertToDbMap(param);
        json.put(CommonConstants.MSG_FIRM_BASE,baseDto);
        json.put(CommonConstants.MSG_FIRM_EXTEND,extendMap);
        Message msg = new Message( //
                // Message所属的Topic
                CommonConstants.MQ_TOPIC,
                // Message Tag 可理解为Gmail中的标签，对消息进行再归类，方便Consumer指定过滤条件在MQ服务器过滤
                CommonConstants.MQ_TAG,
                // Message Key 跟踪消息使用
                baseDto.getClientId(),
                // Message Body 可以是任何二进制形式的数据， MQ不做任何干预
                // 需要Producer与Consumer协商好一致的序列化和反序列化方式
                json.toJSONString().getBytes());
        try {
            SendResult result = producer.send(msg);
            if(result!=null){
                log.info("send message：clientId={} success！",baseDto.getClientId());
            }
        } catch (Exception e) {
            log.error("send message error!msg= {}",msg,e);
        }
    }

    private FirmBaseDto buildBaseDto(FirmParam param) {
        FirmBaseDto dto = new FirmBaseDto();
        BeanUtils.copyProperties(param,dto);
        return dto;
    }

    @Override
    public boolean delete(String clientId) {
        // 逻辑删除可信客户
        firmDao.deleteByClientId(clientId);
        return true;
    }

    /**
     * 执行添加
     *
     * @param param
     * @return
     */
    private boolean doAddTrust(FirmParam param) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus transactionStatus) {
                // 插入firm_info表
                FirmInfo firmInfo = convertToFirmInfo(param, true);
                firmDao.insert(firmInfo);

                // 插入firm_info_extend表
                Map<String, String> propMap = FirmPropUtil.convertToDbMap(param);
                firmExtendDao.batchInsert(param.getClientId(), propMap);

            }
        });
        return true;
    }

    private FirmInfo convertToFirmInfo(FirmParam param, boolean isAdd) {
        FirmInfo firmInfo = new FirmInfo();
        BeanUtils.copyProperties(param, firmInfo);
        if (isAdd) {
            firmInfo.setUniqueid(UniqueIdUtils.genrateUniqueId());
            firmInfo.setStatus(FirmStatusEnum.FIRM_SUB_NORMAL.getCode());

        }
        // 客户属性有几个属性在此表维护，以后要迁移至extend表。
        firmInfo.setSendSms(FirmPropUtil.convertDbValue(FirmPropEnum.SMS_SEND.getProperName(), param.getSmsSend()));
        firmInfo.setStampType(FirmPropUtil.convertDbValue(FirmPropEnum.USER_STAMP_AUTO_SET.getProperName(), param.getUserStampAutoSet()));
        return firmInfo;
    }


    /**
     * 校验业务-新建
     *
     * @param param
     */
    private void checkAddBiz(FirmParam param) throws CodeException {
        //clientId冲突校验
        FirmInfo firmInfo = firmDao.getFrimByClientId(param.getClientId());
        if (null != firmInfo) {
            throw new CodeException(FirmErrEum.CLIENTID_CHECK_ERR.getStatus(),FirmErrEum.CLIENTID_CHECK_ERR.getMessage());
        }
        // OrgCode冲突校验
        List<FirmInfo> list = firmDao.getListByOrgCode(param.getFirmOrgCode());
        if (null != list && list.size() > 0) {
            throw new CodeException(FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR.getStatus(),FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR.getMessage());
        }
        // firmName冲突校验
        list = firmDao.getListByFirmName(param.getFirmName());
        if (null != list && list.size() > 0) {
            throw new CodeException(FirmErrEum.FIRM_NAME_CHECK_ERR.getStatus(),FirmErrEum.FIRM_NAME_CHECK_ERR.getMessage());
        }
    }

    /**
     * 校验业务-更新
     *
     * @param param
     */
    private void checkUpdateBiz(FirmParam param) throws CodeException {

        //clientId冲突校验
        FirmInfo firmInfo = firmDao.getFrimByClientId(param.getClientId());
        if (null == firmInfo) {
            throw new CodeException(FirmErrEum.FIRM_NOT_EXIST.getStatus(),FirmErrEum.FIRM_NOT_EXIST.getMessage());
        }

        // OrgCode冲突校验
        List<FirmInfo> list = firmDao.getListByOrgCode(param.getFirmOrgCode());
        if (list != null && (list.size() > 1 || !Objects.equals(param.getClientId(), list.get(0).getClientId()))) {
            throw new CodeException(FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR.getStatus(),FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR.getMessage());
        }
        // firmName冲突校验
        list = firmDao.getListByFirmName(param.getFirmName());
        if (list != null && (list.size() > 1 || !Objects.equals(param.getClientId(), list.get(0).getClientId()))) {
            throw new CodeException(FirmErrEum.FIRM_NAME_CHECK_ERR.getStatus(),FirmErrEum.FIRM_NAME_CHECK_ERR.getMessage());
        }
    }


    private boolean doUpdateTrust(FirmParam param) {
        int num = transactionTemplate.execute(new TransactionCallback<Integer>() {

            @Override
            public Integer doInTransaction(TransactionStatus transactionStatus) {
                // 插入firm_info表
                FirmInfo firmInfo = convertToFirmInfo(param, false);
                int num = firmDao.update(firmInfo);

                // 插入firm_info_extend表
                Map<String, String> propMap = FirmPropUtil.convertToDbMap(param);
                firmExtendDao.batchInsert(param.getClientId(), propMap);
                return num;
            }
        });

        return num == 1;
    }

    @Override
    public boolean addCaChannelFirm() {
        return false;
    }

    @Override
    public boolean addFirmWhite() {
        return false;
    }

}
