package com.trust.ywx.customer.service.query.impl;

import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.fiftyonetrust.common.redis.template.RedisRepository;
import com.trust.ywx.customer.common.enu.CommonStatusEnum;
import com.trust.ywx.customer.common.enu.FirmPropEnum;
import com.trust.ywx.customer.common.model.PageBean;
import com.trust.ywx.customer.common.model.PageBeanUtil;
import com.trust.ywx.customer.common.model.manager.request.FirmPropParam;
import com.trust.ywx.customer.common.model.manager.response.FirmBaseDto;
import com.trust.ywx.customer.common.model.manager.response.FirmWithExtendDto;
import com.trust.ywx.customer.common.model.query.request.SearchCrmFirmParam;
import com.trust.ywx.customer.common.model.query.request.SearchFirmParam;
import com.trust.ywx.customer.common.model.query.response.CrmFirmDto;
import com.trust.ywx.customer.common.nacos.NacosServer;
import com.trust.ywx.customer.common.util.FirmPropUtil;
import com.trust.ywx.customer.dao.DoctorProductTypeDao;
import com.trust.ywx.customer.dao.FirmDao;
import com.trust.ywx.customer.dao.FirmExtendDao;
import com.trust.ywx.customer.dao.FirmGroupDao;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.dao.po.FirmInfoExtend;
import com.trust.ywx.customer.dao.po.FirmUsercertChannel;
import com.trust.ywx.customer.service.query.FirmQueryService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <pre></pre>
 *
 * @文件名称： FirmQueryServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.query.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/15 18:43
 * @Modify：
 */
@CacheConfig(cacheNames = {"firm"})
@Service("firmQueryService")
public class FirmQueryServiceImpl implements FirmQueryService {
    @Autowired
    private FirmDao firmDao;
    @Autowired
    private FirmGroupDao firmGroupDao;
    @Autowired
    private FirmExtendDao firmExtendDao;
    @Autowired
    private DoctorProductTypeDao doctorProductTypeDao;
    @Autowired
    RedisRepository redisRepository;

    @NacosValue("c")
    private String c;

    @Override
    public PageBean<FirmBaseDto> getList(SearchFirmParam param) {
        final List<String> clientIdList = new ArrayList<>();

        // clientId筛选
        if (StringUtils.isNotEmpty(param.getClientId())) {
            FirmInfo firmInfo = firmDao.getFrimByClientId(param.getClientId());
            if (null == firmInfo) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
            // 增加项目状态的判断
            if (null != param.getProjectStatus() && !Objects.equals(firmInfo.getProjectStatus(), param.getProjectStatus())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
            // 增加实施人员的判断
            if (StringUtils.isNotEmpty(param.getTechId()) && !Objects.equals(firmInfo.getTechPersonId(), param.getTechId())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
            // 增加销售人员的判断
            if (StringUtils.isNotEmpty(param.getSaleId()) && !Objects.equals(firmInfo.getSalePersonId(), param.getTechId())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
            // 增加第三方平台客户id的判断
            if (StringUtils.isNotEmpty(param.getThirdCustomerId()) && !Objects.equals(firmInfo.getSalePersonId(), param.getTechId())) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }

            clientIdList.add(param.getClientId());
        }

        List<FirmInfo> list = firmDao.getListByParam(param);
        if(CollectionUtils.isEmpty(list)){
            return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
        }
        list.stream().forEach(a-> clientIdList.add(a.getClientId()));

        // 客户组筛选
        if (StringUtils.isNotEmpty(param.getGroupId())) {
            List<String> groupClientIdList = firmGroupDao.getClientIdByGroupId(param.getGroupId());
            if (CollectionUtils.isEmpty(clientIdList)) {
                clientIdList.addAll(groupClientIdList);
            } else {
                clientIdList.retainAll(groupClientIdList);
                if (CollectionUtils.isEmpty(clientIdList)) {
                    return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
                }
            }
        }

        // 医师产品形态筛选 fixme 下面的查询可以合成一个
        if (null != param.getDoctorProductType()) {
            List<String> subClientIdList;
            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmPropEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmPropEnum.DOCTOR_PRODUCT_APP.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }

            if (StringUtils.isNotEmpty(param.getClientId())) {
                subClientIdList = getClientIdListByProp(FirmPropEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), Arrays.asList(param.getClientId()));
            } else {
                subClientIdList = getClientIdListByProp(FirmPropEnum.DOCTOR_PRODUCT_SDK.getProperName(), param.getDoctorProductType(), null);
            }
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
        }

        /**
         * 属性过滤暂时没想到好办法，只能一个个去查询了
         */
        if (null != param.getNetworkType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmPropEnum.NETWORK_TYPE.getProperName(), param.getNetworkType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
        }
        if (null != param.getChargingMode()) {
            List<String> subClientIdList = getClientIdListByProp(FirmPropEnum.CHARGING_MODE.getProperName(), param.getChargingMode(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
        }
        if (null != param.getPublicProductType()) {
            List<String> subClientIdList = getClientIdListByProp(FirmPropEnum.PUBLIC_PRODUCT_TYPE.getProperName(), param.getPublicProductType(), clientIdList);
            // 与clientIdList求并集
            clientIdList.retainAll(subClientIdList);
            if (CollectionUtils.isEmpty(clientIdList)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
        }
        PageBean<FirmInfo> page = null;

        if (CollectionUtils.isEmpty(clientIdList)) {
            page = firmDao.getList(param.getPageNo(), param.getPageSize());
        } else {
            if (clientIdList.size() < param.getPageSize() * (param.getPageNo()-1)) {
                return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
            }
            page = firmDao.getListByIds(clientIdList, param.getPageNo(), param.getPageSize());
        }

        if (CollectionUtils.isEmpty(page.getList())) {
            return PageBeanUtil.getEmptyPage(param.getPageNo(), param.getPageSize());
        }
        List<FirmBaseDto> dtoList = new ArrayList<>(page.getList().size());
        for (FirmInfo po : page.getList()) {
            FirmBaseDto dto = converPoToBaseDto(po);
            dtoList.add(dto);
        }

        PageBean pageBean = new PageBean(dtoList);
        pageBean.setPageNum(page.getPageNum());
        pageBean.setPageSize(page.getPageSize());
        pageBean.setTotal(page.getTotal());
        pageBean.setPages(page.getPages());
        return pageBean;
    }


    private FirmBaseDto converPoToBaseDto(FirmInfo po) {
        if (null == po) {
            return null;
        }
        FirmBaseDto dto = new FirmBaseDto();
        BeanUtils.copyProperties(po, dto);

        // TODO 差异属性转换
        return dto;
    }

    @Override
    public FirmBaseDto getBaseByClientId(String clientId) {
        FirmInfo po = firmDao.getFrimByClientId(clientId);
        return converPoToBaseDto(po);
    }

    @Override
    public FirmWithExtendDto getByClientId(String clientId) {
        FirmInfo po = firmDao.getFirmBLOBsByClientId(clientId);
        if (po == null) {
            return null;
        }
        FirmBaseDto baseDto = converPoToBaseDto(po);
        FirmWithExtendDto dto = new FirmWithExtendDto();
        BeanUtils.copyProperties(baseDto, dto);
        Map<String, String> map = getAllProp(clientId);
        FirmPropParam propParam = FirmPropUtil.convertToDto(map);
        dto.setFirmProp(propParam);
        return dto;

    }

    @Override
    public Map<String, String> getAllProp(String clientId) {
        if (StringUtils.isEmpty(clientId)) {
            return new HashMap<>();
        }
        Map<String, String> map = new HashMap<>(64);
        // 1、从firm_info_extend获取全量属性
        List<FirmInfoExtend> extendList = firmExtendDao.getListByClientId(clientId);
        for (FirmInfoExtend extendPo : extendList) {
            map.put(extendPo.getPropName(), extendPo.getPropValue());
        }

        //2、从firm_usercert_channel 获取医师产品形态属性
        List<FirmUsercertChannel> doctorProductList = doctorProductTypeDao.getByClientId(clientId);
        for (FirmUsercertChannel doctorPo : doctorProductList) {
            if (doctorPo.getChannelType() == 0) {
                map.put(FirmPropEnum.DOCTOR_PRODUCT_APP.getProperName(), CommonStatusEnum.YES.getCode());
            }
            if (doctorPo.getChannelType() == 1) {
                map.put(FirmPropEnum.DOCTOR_PRODUCT_SDK.getProperName(), CommonStatusEnum.YES.getCode());
            }
        }
        return map;
    }

    @Override
    public List<FirmBaseDto> getFirmWhite(String firmName) {
        return null;
    }

    @Override
    public List<CrmFirmDto> getCrmFirmList(SearchCrmFirmParam param) {
        return null;
    }

    private List<String> getClientIdListByProp(String propName, Integer propValue, List<String> clientIdList) {
        String convertPropValue = "";
        if (Objects.equals(propName, FirmPropEnum.DOCTOR_PRODUCT_SDK.getProperName())
                || Objects.equals(propName, FirmPropEnum.DOCTOR_PRODUCT_APP.getProperName())) {
            //fixme，目前医师产品形态单独存储在firm_usercert_channel表，后期挪到firm_info_extends表
            return doctorProductTypeDao.getClientIdByProp(clientIdList, propValue);
        } else {
            // 其他属性都在firm_info_extends表
            convertPropValue = String.valueOf(propValue);
            return firmExtendDao.getByPropsAndClientId(propName, convertPropValue, clientIdList);
        }
    }
}
