package com.trust.ywx.customer.service.query.impl;

import com.fiftyonetrust.common.redis.template.RedisRepository;
import com.trust.ywx.customer.common.enu.CommonStatusEnum;
import com.trust.ywx.customer.common.model.PageBean;
import com.trust.ywx.customer.common.model.manager.response.TempFirmDto;
import com.trust.ywx.customer.common.model.query.request.SearchTempFirmParam;
import com.trust.ywx.customer.dao.TempFirmDao;
import com.trust.ywx.customer.dao.po.FirmInfoTemp;
import com.trust.ywx.customer.service.query.TempFirmQueryService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

/**
 * <pre></pre>
 *
 * @文件名称： FirmQueryServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.query.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/15 18:43
 * @Modify：
 */
@CacheConfig(cacheNames = {"firm"})
@Service("tempFirmQueryService")
public class TempFirmQueryServiceImpl implements TempFirmQueryService {
    @Autowired
    private TempFirmDao tempFirmDao;
    @Autowired
    RedisRepository redisRepository;

    @Override
    public PageBean<TempFirmDto> getTempFirmList(SearchTempFirmParam param) {
        return tempFirmDao.getListByParam(param);
    }

    @Override
    @Cacheable(key = "'temp:'+#clientId")
    public TempFirmDto getTempFirmByClientId(String clientId) {
        FirmInfoTemp tempPo = tempFirmDao.getByClientId(clientId);
        TempFirmDto dto = converTempPoToDto(tempPo);
        return dto;
    }

    private TempFirmDto converTempPoToDto(FirmInfoTemp tempPo) {
        TempFirmDto dto = new TempFirmDto();
        BeanUtils.copyProperties(tempPo, dto);
        if (StringUtils.isNotBlank(tempPo.getChannel())) {
            if ("0".equals(tempPo.getChannel())) {
                dto.setChannelApp(CommonStatusEnum.YES.getCode());
            } else if ("1".equals(tempPo.getChannel())) {
                dto.setChannelSdk(CommonStatusEnum.YES.getCode());
            } else if ("2".equals(tempPo.getChannel())) {
                dto.setChannelApp(CommonStatusEnum.YES.getCode());
                dto.setChannelSdk(CommonStatusEnum.YES.getCode());
            }
        }
        return dto;
    }

}
