package com.trust.ywx.customer.service.client;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.*;
import com.fiftyonetrust.common.util.JsonUtil;
import com.trust.ywx.customer.common.CommonConstants;
import com.trust.ywx.customer.common.model.manager.response.FirmBaseDto;
import com.trust.ywx.customer.common.nacos.NacosServer;
import com.trust.ywx.customer.service.query.FirmQueryService;
import lombok.Data;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

/**
 * <pre></pre>
 *
 * @文件名称： CustomerClientService.java
 * @包 路  径： com.trust.ywx.customer.client
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description:
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/28 14:40
 * @Modify：
 */
@Slf4j
@Data
public class FirmLocalCacheService {

    private String rocketMqHost;

    private String accessKey;
    private String secretKey;
    private String groupID;
    private int ConsumeThreadNums = 10;
    private int MaxReconsumeTimes = 5;

    @Resource
    private FirmQueryService firmQueryService;
    @Resource
    private NacosServer nacosServer;

    private static Map<String, FirmBaseDto> localFirmInfoMap = new ConcurrentHashMap<>();
    private static Map<String, Map<String, String>> localFirmExtendMap = new ConcurrentHashMap<>();

    /**
     * 获取可信客户详情
     */
    public FirmBaseDto getFirmByClientId(String clientId) {
        FirmBaseDto dto = null;
        if(nacosServer.getFirmNacosConfig().isUseLocalCache()){
            dto = localFirmInfoMap.get(clientId);
            if (dto != null) {
                return dto;
            }
        }

        try {
            dto = firmQueryService.getBaseByClientId(clientId);
        } catch (Exception e) {
            // 客户端不要打太多日志！
            log.error("firmQueryService.getBaseByClientId error，{}", e.getMessage());
        }
        if (null != dto) {
            localFirmInfoMap.put(clientId, dto);
        }
        return dto;
    }

    /**
     * 获取客户属性信息
     *
     * @param clientId
     * @param propName
     * @return
     */
    public String getProp(String clientId, String propName) {
        Map<String, String> extendMap = localFirmExtendMap.get(clientId);
        if (null != extendMap) {
            return extendMap.get(propName);
        }

        // 调用dubbo接口获取属性信息
        try {
            extendMap = firmQueryService.getAllProp(clientId);
        } catch (Exception e) {
            log.error("firmQueryService.getAllProp error，{}", e.getMessage());
        }
        if (extendMap != null) {
            localFirmExtendMap.put(clientId, extendMap);
        }
        return null;
    }

    private void refresh(FirmBaseDto dto) {
        localFirmInfoMap.put(dto.getClientId(), dto);
    }

    private void refresh(Map<String, String> map, String clientId) {
        localFirmExtendMap.put(clientId, map);
    }

    @Bean
    public void initConsumer() {
        Properties properties = new Properties();
        properties.put("ONSAddr", nacosServer.getFirmNacosConfig().getRocketMqNameServAddr());
        properties.put("AccessKey", nacosServer.getFirmNacosConfig().getAliAccessKey());
        properties.put("SecretKey", nacosServer.getFirmNacosConfig().getAliSecretKey());
         properties.put("ConsumerId", "ywx-customer");
        properties.setProperty("ConsumeThreadNums", String.valueOf(ConsumeThreadNums));
        properties.setProperty("maxReconsumeTimes", String.valueOf(MaxReconsumeTimes));
        // 广播订阅方式设置。
        properties.put(PropertyKeyConst.MessageModel, PropertyValueConst.BROADCASTING);
        Consumer consumer = ONSFactory.createConsumer(properties);

        consumer.subscribe(CommonConstants.MQ_TOPIC, CommonConstants.MQ_TAG, new MessageListener() {
            @SneakyThrows
            @Override
            public Action consume(Message message, ConsumeContext consumeContext) {
                JSONObject json = JSONObject.parseObject(new String(message.getBody(), "utf-8"));
                FirmBaseDto dto = null;
                try {
                    dto = JSONObject.parseObject(CommonConstants.MSG_FIRM_BASE, FirmBaseDto.class);
                    refresh(dto);
                } catch (Exception e) {
                    log.warn("consume handled firmBase error,{}", json);
                }
                try {
                    Map<String, String> map = JsonUtil.ofMap(CommonConstants.MSG_FIRM_EXTEND, String.class, String.class);
                    refresh(map, dto.getClientId());
                    return Action.CommitMessage;
                } catch (Exception e) {
                    log.warn("consume handled firmExtends error,{}", json);
                    return Action.ReconsumeLater;
                }
            }
        });

        consumer.start();
        log.info(" MQ consumer started, topic = {} ", CommonConstants.MQ_TOPIC);
        Thread shutdownHookOne = new Thread() {
            public void run() {
                log.info(" MQ Consumer shutdown!");
                if (consumer != null) {
                    consumer.shutdown();
                    log.info(" MQ Consumer shutdown success!");
                }
            }
        };
        Runtime.getRuntime().addShutdownHook(shutdownHookOne);
    }

}
